//  BookLibraryAppDelegate.m
//  BookLibrary
//
//  Created by KOYAMA Yoshiaki on 05/10/02.
//  Copyright KOYAMA Yoshiaki 2005 . All rights reserved.

#import "BookLibraryAppDelegate.h"

@implementation BookLibraryAppDelegate

- (NSManagedObjectModel *)managedObjectModel {
    if (managedObjectModel) return managedObjectModel;
	
	NSMutableSet *allBundles = [[NSMutableSet alloc] init];
	[allBundles addObject: [NSBundle mainBundle]];
	[allBundles addObjectsFromArray: [NSBundle allFrameworks]];
    
    managedObjectModel = [[NSManagedObjectModel mergedModelFromBundles: [allBundles allObjects]] retain];
    [allBundles release];
    
    return managedObjectModel;
}

/* Change this path/code to point to your App's data store. */
- (NSString *)applicationSupportFolder {
    NSString *applicationSupportFolder = nil;
    FSRef foundRef;
    OSErr err = FSFindFolder(kUserDomain, kApplicationSupportFolderType, kDontCreateFolder, &foundRef);
    if (err != noErr) {
        NSRunAlertPanel(@"Alert", @"Can't find application support folder", @"Quit", nil, nil);
        [[NSApplication sharedApplication] terminate:self];
    } else {
        unsigned char path[1024];
        FSRefMakePath(&foundRef, path, sizeof(path));
        applicationSupportFolder = [NSString stringWithUTF8String:(char *)path];
        applicationSupportFolder = [applicationSupportFolder stringByAppendingPathComponent:@"BookLibrary"];
    }
    return applicationSupportFolder;
}

- (NSManagedObjectContext *) managedObjectContext {
    NSError *error;
    NSString *applicationSupportFolder = nil;
    NSURL *url;
    NSFileManager *fileManager;
    NSPersistentStoreCoordinator *coordinator;
    
    if (managedObjectContext) {
        return managedObjectContext;
    }
    
    fileManager = [NSFileManager defaultManager];
    applicationSupportFolder = [self applicationSupportFolder];
    if ( ![fileManager fileExistsAtPath:applicationSupportFolder isDirectory:NULL] ) {
        [fileManager createDirectoryAtPath:applicationSupportFolder attributes:nil];
    }
    
    url = [NSURL fileURLWithPath: [applicationSupportFolder stringByAppendingPathComponent: @"BookLibrary.xml"]];
    coordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel: [self managedObjectModel]];
    if ([coordinator addPersistentStoreWithType:NSXMLStoreType configuration:nil URL:url options:nil error:&error]){
        managedObjectContext = [[NSManagedObjectContext alloc] init];
        [managedObjectContext setPersistentStoreCoordinator: coordinator];
    } else {
        [[NSApplication sharedApplication] presentError:error];
    }    
    [coordinator release];
    
    return managedObjectContext;
}

- (NSUndoManager *)windowWillReturnUndoManager:(NSWindow *)window {
    return [[self managedObjectContext] undoManager];
}

- (IBAction) saveAction:(id)sender {
    NSError *error = nil;
    if (![[self managedObjectContext] save:&error]) {
        [[NSApplication sharedApplication] presentError:error];
    }
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender {
    NSError *error;
    NSManagedObjectContext *context;
    int reply = NSTerminateNow;
    
    context = [self managedObjectContext];
    if (context != nil) {
        if ([context commitEditing]) {
            if (![context save:&error]) {
				
				// This default error handling implementation should be changed to make sure the error presented includes application specific error recovery. For now, simply display 2 panels.
                BOOL errorResult = [[NSApplication sharedApplication] presentError:error];
				
				if (errorResult == YES) { // Then the error was handled
					reply = NSTerminateCancel;
				} else {
					
					// Error handling wasn't implemented. Fall back to displaying a "quit anyway" panel.
					int alertReturn = NSRunAlertPanel(nil, @"Could not save changes while quitting. Quit anyway?" , @"Quit anyway", @"Cancel", nil);
					if (alertReturn == NSAlertAlternateReturn) {
						reply = NSTerminateCancel;	
					}
				}
            }
        } else {
            reply = NSTerminateCancel;
        }
    }
    return reply;
}

- (IBAction)input:(id)sender {

	[self clearTextField];

	[NSApp beginSheet:_panel 
	modalForWindow:window 
	modalDelegate:self
	didEndSelector:@selector(sheetDidEnd:returnCode:contextInfo:)
	contextInfo:nil];
}

- (void)sheetDidEnd:(NSWindow *)sheet 
            returnCode:(int)returnCode
            contextInfo:(void *)contextInfo {

    if (returnCode == 1);
}

- (void)add:(id)sender {
	id obj;

    [_panel orderOut:sender];
    [NSApp endSheet:_panel returnCode:1];

    obj = [NSEntityDescription insertNewObjectForEntityForName:@"Book" inManagedObjectContext:[self managedObjectContext]];

	[obj setValue:[_packageDimensions stringValue] forKey:@"packageDimensions"];
	[obj setValue:[_title stringValue] forKey:@"title"];
	[obj setValue:[_isbn stringValue] forKey:@"isbn"];
	[obj setValue:[NSNumber numberWithInt:[_amount intValue]] forKey:@"amount"];
	[obj setValue:[[_image image] TIFFRepresentation] forKey:@"image"];
	[obj setValue:[_publicationDate stringValue] forKey:@"publicationDate"];
	[obj setValue:[NSNumber numberWithInt:[_numberOfPages intValue]] forKey:@"numberOfPages"];
	[obj setValue:[_binding stringValue] forKey:@"binding"];
	[obj setValue:[NSNumber numberWithInt:[_salesRank intValue]] forKey:@"salesRank"];
	[obj setValue:[_publisher stringValue] forKey:@"publisher"];
	[obj setValue:[_creator stringValue] forKey:@"creator"];
	[obj setValue:[_author stringValue] forKey:@"author"];
	[obj setValue:[_comments stringValue] forKey:@"comments"];

	[obj setValue:[NSDate date] forKey:@"entryDate"];

}

- (IBAction)cancel:(id)sender {
    [_panel orderOut:sender];
    [NSApp endSheet:_panel returnCode:2];
	[_indicator stopAnimation:self];
}

- (IBAction)amazon:(id)sender {
	NSXMLDocument *xmlDoc;
	NSError *err = nil;
	NSURL *url;
	int i;
	NSArray *nodes;
	NSString *urlStr;
//	NSString *itemID = @"4839918805";
//	NSString *itemID = @"4875934289";
//	NSString *itemID = @"4861001242";
//	NSString *itemID = @"4873110815";
	NSString *imageStr;
	NSImage *image;
	NSMutableString *creatorRoleStr, *authorRoleStr, *packageRoleStr;
	NSString *creatorStr, *authorStr, *roleStr, *nodeStr, *itemID, *packageStr;
	int count;

	itemID = [_isbn stringValue];

	if ((itemID == nil) || ([itemID isEqualToString:@""])) {
//		itemID = @"477411393X";
//		itemID = @"4873110815";
//		itemID = @"4339023167";
//		itemID = @"4873112451"; //{[V
//		[_isbn setStringValue:itemID];
		return;
	}

	[_indicator startAnimation:self];
//	NSLog(@"%@", itemID);

//	ftHgݒ
//	urlStr = [NSString stringWithFormat:@"http://webservices.amazon.co.jp/onca/xml?Service=AWSECommerceService&SubscriptionId=%@&AssociateTag=%@&Operation=ItemLookup&ItemId=%@&ResponseGroup=Medium", SUBSCRIPTION_ID, ASSOCIATE_TAG, itemID];

//	ȗ
	urlStr = [NSString stringWithFormat:@"http://webservices.amazon.co.jp/onca/xml?Service=AWSECommerceService&SubscriptionId=%@&AssociateTag=%@&Operation=ItemLookup&ResponseGroup=Request,Images,ItemAttributes,SalesRank&ContentType=text/xml&Version=2005-10-05&ItemId=%@", SUBSCRIPTION_ID, ASSOCIATE_TAG, itemID];
//	urlStr = [NSString stringWithFormat:@"http://webservices.amazon.co.jp/onca/xml?Service=AWSECommerceService&SubscriptionId=%@&AssociateTag=%@&Operation=ItemLookup&ResponseGroup=Request,Images,ItemAttributes,SalesRank&ContentType=text/xml&Version=2005-09-15&ItemId=%@", SUBSCRIPTION_ID, ASSOCIATE_TAG, itemID];

//	NSLog(@"%@", urlStr);

	url = [NSURL URLWithString:urlStr];
//	options0łȂƓ삪Ȃ
	xmlDoc = [[NSXMLDocument alloc] initWithContentsOfURL:url options:0 error:&err];

//	xmlDoc = [[NSXMLDocument alloc] initWithContentsOfURL:url options:(NSXMLNodePreserveWhitespace|NSXMLNodePreserveCDATA) error:&err];
//	xmlDoc = [[NSXMLDocument alloc] initWithContentsOfURL:url options:NSXMLDocumentTidyXML error:&err];

//   [NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.5]];

	nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Request/Errors/Error" error:&err];
//	NSLog(@"%d", [nodes count]);

	if ((xmlDoc == nil) || ([nodes count] != 0)){
		[_indicator stopAnimation:self];
		return;
	}  

// XML \
/*	NSArray *children;
	NSXMLNode *child;
	children = [[xmlDoc rootElement] children];
	count = [children count];
	
	for (i=0; i < count; i++) {
		child = [children objectAtIndex:i];
		NSLog(@"%@", child);
	}
*/

//	packageDimensions
	packageRoleStr = [NSMutableString string];
	nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/ItemAttributes/PackageDimensions/Length" error:&err];
	count = [nodes count];

	if (count != 0) {
		packageStr = [[nodes objectAtIndex:0] stringValue];
		roleStr = [[[[nodes objectAtIndex:0] nodesForXPath:@"@Units" error:&err] objectAtIndex:0] stringValue];
		[packageRoleStr appendFormat:@"Length:%@%@", packageStr, roleStr];

		nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/ItemAttributes/PackageDimensions/Width" error:&err];
		count = [nodes count];

		if (count != 0) {
			packageStr = [[nodes objectAtIndex:0] stringValue];
			roleStr = [[[[nodes objectAtIndex:0] nodesForXPath:@"@Units" error:&err] objectAtIndex:0] stringValue];
			[packageRoleStr appendFormat:@" Width:%@%@", packageStr, roleStr];
		}

		[_packageDimensions setStringValue:packageRoleStr];
	}

//	nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/ASIN" error:&err];
	nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/ItemAttributes/Title" error:&err];
	if ([nodes count] != 0) {
		[_title setStringValue:[[nodes objectAtIndex:0] stringValue]];
//		NSLog(@"title =%@", [[nodes objectAtIndex:0] stringValue]);
	}

	nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/ItemAttributes/ListPrice/Amount" error:&err];
	if ([nodes count] != 0) {
		nodeStr = [[nodes objectAtIndex:0] stringValue];
//		NSLog(@"amount=%@", nodeStr);
		[_amount setIntValue:[nodeStr intValue]];
	}

//	image
	nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/MediumImage/URL" error:&err];
	if ([nodes count] == 0) {
//		[NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:1.8]];
//		NSLog(@"nodes count 0");
//		nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/MediumImage/URL" error:&err];
		nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/ImageSets/ImageSet/MediumImage/URL" error:&err];
	}

//	NSLog(@"node = %d", [nodes count]);
	if ([nodes count] != 0) {
//		NSLog(@"%@", nodes);

		imageStr = [[nodes objectAtIndex:0] stringValue];
//		NSLog(@"url =%@", imageStr);

		image = [[NSImage alloc] initWithContentsOfURL:[NSURL URLWithString:imageStr]];
		[_image setImage:image];
	}
	
	nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/ItemAttributes/PublicationDate" error:&err];
	if ([nodes count] != 0) {
//		NSLog(@"publicationDate =%@", [[nodes objectAtIndex:0] stringValue]);
		[_publicationDate setStringValue:[[nodes objectAtIndex:0] stringValue]];
	}

	nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/ItemAttributes/NumberOfPages" error:&err];
	if ([nodes count] != 0) {
		nodeStr = [[nodes objectAtIndex:0] stringValue];
//		NSLog(@"amount=%@", nodeStr);
		[_numberOfPages setIntValue:[nodeStr intValue]];
	}

	nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/ItemAttributes/Binding" error:&err];
	if ([nodes count] != 0) {
//		NSLog(@"binding=%@", [[nodes objectAtIndex:0] stringValue]);
		[_binding setStringValue:[[nodes objectAtIndex:0] stringValue]];
	}

	nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/SalesRank" error:&err];
	if ([nodes count] != 0) {
		nodeStr = [[nodes objectAtIndex:0] stringValue];
//		NSLog(@"salesRank=%@", nodeStr);
		[_salesRank setIntValue:[self intValue:nodeStr]];
	}

	nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/ItemAttributes/Publisher" error:&err];
	if ([nodes count] != 0) {
//		NSLog(@"publisher =%@", [[nodes objectAtIndex:0] stringValue]);
		[_publisher setStringValue:[[nodes objectAtIndex:0] stringValue]];
	}
	
	creatorRoleStr = [NSMutableString string];
	nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/ItemAttributes/Creator" error:&err];
	count = [nodes count];

	if (count != 0) {
		for (i=0; i<count; i++) {
			creatorStr = [[nodes objectAtIndex:i] stringValue];
			roleStr = [[[[nodes objectAtIndex:i] nodesForXPath:@"@Role" error:&err] objectAtIndex:0] stringValue];

//			NSLog(@"creater=%@", creatorStr);
//			NSLog(@"creater role=%@", roleStr);
	
			if (i == (count-1)) {
				[creatorRoleStr appendFormat:@"%@ %@", creatorStr, roleStr];
			} else {
				[creatorRoleStr appendFormat:@"%@ %@, ", creatorStr, roleStr];
			}
		}
		[_creator setStringValue:creatorRoleStr];
	}


//	author
	authorRoleStr = [NSMutableString string];
	nodes = [xmlDoc nodesForXPath:@".//ItemLookupResponse/Items/Item/ItemAttributes/Author" error:&err];
	count = [nodes count];

	if (count != 0) {
		for (i=0; i<count; i++) {
			authorStr = [[nodes objectAtIndex:i] stringValue];
			if (i == (count-1)) {
				[authorRoleStr appendFormat:@"%@", authorStr];
			} else {
				[authorRoleStr appendFormat:@"%@, ", authorStr];
			}
//			NSLog(@"author =%@", [[nodes objectAtIndex:i] stringValue]);
		}
		[_author setStringValue:authorRoleStr];
	}

	[xmlDoc release];
	[_indicator stopAnimation:self];
}

- (void)awakeFromNib {
    NSFont *aFont;
    NSTableColumn *column;
	NSCell *cell;

	aFont = [NSFont fontWithName:@"HiraKakuPro-W3" size:10];
			
    column = [_tableView tableColumnWithIdentifier:@"Title"];
    cell = [column dataCell];
	[cell setFont:aFont];
	
    column = [_tableView tableColumnWithIdentifier:@"Creator"];
    cell = [column dataCell];
	[cell setFont:aFont];

    column = [_tableView tableColumnWithIdentifier:@"Publisher"];
    cell = [column dataCell];
	[cell setFont:aFont];

    column = [_tableView tableColumnWithIdentifier:@"PublicationDate"];
    cell = [column dataCell];
	[cell setFont:aFont];
/*
    column = [_tableView tableColumnWithIdentifier:@"Isbn"];
    cell = [column dataCell];
	[cell setFont:aFont];
*/
    column = [_tableView tableColumnWithIdentifier:@"EntryDate"];
    cell = [column dataCell];
	[cell setFont:aFont];

    column = [_tableView tableColumnWithIdentifier:@"Binding"];
    cell = [column dataCell];
	[cell setFont:aFont];

    column = [_tableView tableColumnWithIdentifier:@"Comments"];
    cell = [column dataCell];
	[cell setFont:aFont];

    column = [_tableView tableColumnWithIdentifier:@"Amount"];
    cell = [column dataCell];
	[cell setFont:aFont];
}

// EChE̏
- (BOOL)windowShouldClose:(id)sender {
    [NSApp terminate:self];
    return YES;
}

- (int)intValue:(NSString *)str {
	NSString *cStr;
	NSMutableString *mStr;
    NSScanner *scan;
    NSCharacterSet *set;
	BOOL flag = NO;

	mStr = [NSMutableString string];
    scan = [NSScanner scannerWithString:str];
    set = [NSCharacterSet characterSetWithCharactersInString:@","];

    while ([scan isAtEnd] == NO) {
        flag = [scan scanUpToCharactersFromSet:set intoString:&cStr];
		if (flag == YES) {
//			NSLog(@"string %@", cStr);
			[mStr appendString:cStr];
			[scan scanString:@"," intoString:NULL];
		}
	}

//	NSLog(@"string %@ : %@", str, mStr);

	return [mStr intValue];
}

- (void)clearTextField {
	[_packageDimensions setStringValue:@""];
	[_title setStringValue:@""];
	[_isbn setStringValue:@""];
	[_amount setIntValue:0];
	[_image setImage:nil];
	[_publicationDate setStringValue:@""];
	[_numberOfPages setIntValue:0];
	[_binding setStringValue:@""];
	[_salesRank setIntValue:0];
	[_publisher setStringValue:@""];
	[_creator setStringValue:@""];
	[_author setStringValue:@""];
	[_comments setStringValue:@""];
}

@end
